//*******************************************************
//*    		init.c : source for initialization     		*
//*						pic16f690		 	  		 	*
//*						MPLAB V8.86				 	    *
//* 					PICC lite 					    *
//* 				Simon Chaudeurge				    *
//* 					August 2012					    *
//*******************************************************

#include "init.h"
#include "lcd.h"
#include "pic.h"
#include "common.h"
#include "input.h"
#include "shoot.h"

//***********************************************
// vInit function : Init uC and pin config
//***********************************************
void vInit(void)
{
	//general setting
  	OSCCON = 0x70;         // 0x70 Select 8 MHz internal clock

    T2CON  = 0x01;  // Turn off Timer2, prescale 1:4, postscale 1:1
    CM1CON0 = 0x00; // Turn off comparator 1
    CM2CON0 = 0x00; // Turn off comparator 2
   
   	vInitVariable();
	vInitInterrupt();
	vInitPortDir();
	vInitUsedPin();
	vInitUnusedPin();
	vInitTimer();
	vInitLCD();
	vInitADC();
}

//***********************************************
// vInitVariable function : Init globale variables
//***********************************************
void vInitVariable(void)
{
	uiShootInterval_sec = 1;
	ulTotalShootTime_min = 1;
	ucLCDstateOld = 0;
	ucButtonPressed = 0;
	ucSequenceTimeout = 0;
	ulNumberOfPhotoTaken = 0;
	ulTotalNumberOfPhoto = 0;
}
	
//***********************************************
// vInitADC function : Init ADC config
//		ADC is used to read the 5 switch button:
//***********************************************
void vInitADC(void)
{
	// Define I/O or AN pin
	ANSEL = 0b00000100;	// RA2(AN2) set as input Analog input other as I/O port
	ANSELH = 0b00000000;

	// Select Channel 2 (AN2)
	CHS0=0;	
	CHS1=1;
	CHS2=0;
	CHS3=0;

	// ADCON0
	ADFM = 1; //format Right justified (ADCON0 setting)
	VCFG = 0;	//Vref=Vdd (ADCON0 setting)
	ADON = 1; //enable ADC module

	ADCON1=0x00; //ADCS conversion clock = FOSC/2 (4MHz)
}


//***********************************************
// vInitInterrupt function : Init the interruption 
//		(used with the timer)
//***********************************************
void vInitInterrupt(void)
{
	INTCON = 0x00;  // disable all interrupt sources
	PIE1 = 0x00;	//turn off peripheral interrupt
	PIE2 = 0x00;	//turn off peripheral interrupt
}


//***********************************************
// vInitPortDir function : Init port direction 
// 		0 = output
//		1 = input
//***********************************************
void vInitPortDir(void)
{
	TRISA0 = 0;	// ICSPDAT
	TRISA1 = 0;	// ICSPCLK
	TRISA2 = 1;	// input switch (ADC)
	TRISA3 = 0;	// MCLR
	TRISA4 = 0;	// UNUSED
	TRISA5 = 0;	// UNUSED

	TRISB4 = 0;	// SHOOT pin direction (output)
	TRISB5 = 0; // UNUSED
	TRISB6 = 0;	// FOCUS pin direction (output)
	TRISB7 = 0; // UNUSED

	TRISC0 = 0; // LCD data1
	TRISC1 = 0; // LCD data2
	TRISC2 = 0; // LCD data3
	TRISC3 = 0; // LCD data4
	TRISC4 = 0;	// UNUSED
	TRISC5 = 0; // LCD RS
	TRISC6 = 0; // UNUSED
	TRISC7 = 0; // LCD Enable
}

//***********************************************
// vInitUsedPin function : Init pin used
//***********************************************
void vInitUsedPin(void)
{
	FOCUS = 0;
	SHOOT = 0;
	PORTC = 0x00;
}

//***********************************************
// vInitUnusedPin function : Init pin unused
//***********************************************
void vInitUnusedPin(void)
{	
	RA4 = 0;
	RA5 = 0;
	RB5 = 0;
	RB7 = 0;
	RC4 = 0;
	RC6 = 0;
}

//***********************************************
// vInitTimer : Init the Timer1 used to trig shot
//***********************************************
void vInitTimer(void)
{
	/* Init TIMER2:  
	Period = Fosc/4 x Prescaler x PR2 x Postscal :
		   = 0.0005ms * 16 * 252 * 16 = 32,256ms
	*/
	T2CON   = 0xFF;       //Prescal = postcal = 16 
	PR2     = 0xFC;        
	TMR2ON  = 1;
}



//TODO: a utiliser :

//***********************************************
// vEnableInterrupt function : Enable the interruption 
//		(used with the timer)
//***********************************************
void vEnableTimerInterrupt(void)
{
	TMR2IE = 1; // Enable interrupt on TMR2 (TMR2IF)
	PEIE = 1; //Enables all unmasked peripheral interrupts
	GIE = 1; // Enables all unmasked interrupts
}

//***********************************************
// vDisableInterrupt function : Disable the interruption 
//		(used with the timer)
//***********************************************
void vDisableTimerInterrupt(void)
{
	TMR2IE = 0; // Enable interrupt on TMR2 (TMR2IF)
}