//*******************************************************
//*    		input.c : source for input button switch	*
//*						pic16f690		 	  		 	*
//*						MPLAB V8.86				 	    *
//* 					PICC lite 					    *
//* 				Simon Chaudeurge				    *
//* 					August 2012					    *
//*******************************************************

#include "input.h"
#include "pic.h"
#include "lcd.h"
#include "common.h"

//***********************************************
//	Variables
//***********************************************
unsigned char ucDebounceSelect = 0x00;
unsigned char ucDebounceLeft = 0x00;
unsigned char ucDebounceRight = 0x00;
unsigned char ucDebounceUp = 0x00;
unsigned char ucDebounceDown = 0x00;
unsigned char ucDebounceNone = 0x00;
unsigned char ucButtonPressedTmp = 0x00;



//***********************************************
// vReadInputSw function : Read pressed button
//		based on a ADC conversion
// input: None
// output: ucButtonPressed=	NONE
//							SELECT
//							LEFT
//							RIGHT
//							UP
//							DOWN							
//***********************************************
void vReadInputSw(void)
{
	unsigned short usResult = 0;
	unsigned char ucResult = 0;
	unsigned char i = 0;

	//calculate the average of 4 conversions
	for (i=0; i<4;i++)	usResult += usReadADC(); 
	usResult = usResult >> 2;


	//detect the button pressed according to the ADC result
	//Detect RIGHT button
	if(usResult < SW_RIGHT_THRESHOLD_MAX)
	{
		if(ucDebounceRight != 0xFF) ucDebounceRight++;
		ucDebounceLeft = 0x00;
		ucDebounceSelect = 0x00;
		ucDebounceUp = 0x00;
		ucDebounceDown = 0x00;
		ucDebounceNone = 0x00;
	}

	//Detect UP button
	else if((usResult > SW_UP_THRESHOLD_MIN)
		&& (usResult < SW_UP_THRESHOLD_MAX))
	{
		if(ucDebounceUp != 0xFF) ucDebounceUp++;
		ucDebounceLeft = 0x00;
		ucDebounceRight = 0x00;
		ucDebounceSelect = 0x00;
		ucDebounceDown = 0x00;
		ucDebounceNone = 0x00;
	}

	//Detect DOWN button
	else if((usResult > SW_DOWN_THRESHOLD_MIN)
		&& (usResult < SW_DOWN_THRESHOLD_MAX))
	{
		if(ucDebounceDown != 0xFF) ucDebounceDown++;
		ucDebounceLeft = 0x00;
		ucDebounceRight = 0x00;
		ucDebounceUp = 0x00;
		ucDebounceSelect = 0x00;
		ucDebounceNone = 0x00;
	}

	//Detect LEFT button
	else if((usResult > SW_LEFT_THRESHOLD_MIN)
		&& (usResult < SW_LEFT_THRESHOLD_MAX))
	{
		if(ucDebounceLeft != 0xFF) ucDebounceLeft++;
		ucDebounceSelect = 0x00;
		ucDebounceRight = 0x00;
		ucDebounceUp = 0x00;
		ucDebounceDown = 0x00;
		ucDebounceNone = 0x00;
	}

	//Detect Select button
	else if((usResult > SW_SELECT_THRESHOLD_MIN)
		&& (usResult < SW_SELECT_THRESHOLD_MAX))
	{
		if(ucDebounceSelect != 0xFF) ucDebounceSelect++;
		ucDebounceLeft = 0x00;
		ucDebounceRight = 0x00;
		ucDebounceUp = 0x00;
		ucDebounceDown = 0x00;
		ucDebounceNone = 0x00;
	}

	//No button pressed
	else if (usResult > SW_NONE_THRESHOLD_MIN)
	{
		//if(ucDebounceNone != 0xFF) ucDebounceNone++;
		ucDebounceNone = 0x01;	//No debounce requiered for release button
		ucDebounceSelect = 0x00;
		ucDebounceLeft = 0x00;
		ucDebounceRight = 0x00;
		ucDebounceUp = 0x00;
		ucDebounceDown = 0x00;
	}
	

	//1) Valid a button pressed when its release 
	if(ucDebounceSelect >= THRESHOLD_BUTTON)
	{
		ucButtonPressedTmp = SELECT;
	}
	else if (ucDebounceLeft >= THRESHOLD_BUTTON)
	{
		ucButtonPressedTmp = LEFT;
	}
	else if (ucDebounceRight >= THRESHOLD_BUTTON)
	{
		ucButtonPressedTmp = RIGHT;
	}
	else if (ucDebounceUp >= THRESHOLD_BUTTON)
	{
		ucButtonPressedTmp = UP;
	}
	else if (ucDebounceDown >= THRESHOLD_BUTTON)
	{
		ucButtonPressedTmp = DOWN;
	}
//	else if (ucDebounceNone >= THRESHOLD_BUTTON)
	else if (ucDebounceNone)
	{
		ucButtonPressed = 	ucButtonPressedTmp;
		ucButtonPressedTmp = NONE;		
	}

	//2) Also valid a button after a long time pressing it
		if(ucDebounceSelect >= THRESHOLD_BUTTON_LONG)
	{
		ucButtonPressed = SELECT;
	}
	else if (ucDebounceLeft >= THRESHOLD_BUTTON_LONG)
	{
		ucButtonPressed = LEFT;
	}
	else if (ucDebounceRight >= THRESHOLD_BUTTON_LONG)
	{
		ucButtonPressed = RIGHT;
	}
	else if (ucDebounceUp >= THRESHOLD_BUTTON_LONG)
	{
		ucButtonPressed = UP;
	}
	else if (ucDebounceDown >= THRESHOLD_BUTTON_LONG)
	{
		ucButtonPressed = DOWN;
	}
	
	__delay_ms(1);
}

//***********************************************
// vReadADC function : Get the result of the 
//	ADC conversion
//***********************************************
unsigned short usReadADC(void)
{
	unsigned short usResultADC = 0;
	unsigned char ucResultVoltage = 0;

	GO_DONE=1;	//Start conversion  
	while(GO_DONE!=0) 
	{
		__delay_us(1);
	}
		usResultADC = (ADRESH << 8) + ADRESL;
		//Wait minimum 2*Tad before a next acquisition

	return usResultADC;
}