//*******************************************************
//*    		lcd.c : source for use of LCD	     		*
//*				LCD1602  16x2char 4bits mode			*
//*						pic16f690		 	  		 	*
//*						MPLAB V8.86				 	    *
//* 					PICC lite 					    *
//* 				Simon Chaudeurge				    *
//* 					August 2012					    *
//*******************************************************

#include "pic.h"
#include "lcd.h"
#include "input.h"
#include "shoot.h"
#include "common.h"
#include "init.h"
#include "modes.h"

unsigned char ucLCDstate = 0x00;
unsigned char ucMode = 0x00;
static	unsigned long milliard = 0;
static	unsigned long million = 0;
static	unsigned short mil = 0;
static	unsigned int cent = 0;
static	unsigned int diz = 0;
static	unsigned int unit = 0; 

//***********************************************
// Basic LCD functions
//***********************************************
void vInitLCD(void)
{
	vResetLCD();         // Call LCD reset
    vLCDsendCmd(0x28);       // 4-bit mode - 2 line - 5x7 font.
    vLCDsendCmd(0x0C);       // Display no cursor - no blink.
    vLCDsendCmd(0x06);       // Automatic Increment - No Display shift.
    vLCDsendCmd(0x80);       // Address DDRAM with 0 offset 80h.
}

void vResetLCD(void)
{
	__delay_ms(30);

	DATA = (0x03 | EN);
	__delay_us(1);
	DATA = (0x03);
	__delay_ms(5);

	DATA = (0x03 | EN);
	__delay_us(1);
	DATA = (0x03);
	__delay_us(200);

	DATA = (0x03 | EN);
	__delay_us(1);
	DATA = (0x03);
	__delay_ms(200);

	DATA = (0x02 | EN);
	__delay_us(1);
	DATA = (0x02);
	__delay_ms(5);

	vLCDclear();
}


void  vLCDsendCmd(char cmd)
{
	DATA = (((cmd >> 4) & 0x0F) | EN);
	__delay_us(1);
	DATA = ((cmd >> 4) & 0x0F);
	__delay_ms(5);
	
	DATA = ((cmd & 0x0F) | EN);
	__delay_us(1);
	DATA = (cmd & 0x0F);
	__delay_ms(5);
}

void vLCDsendData(unsigned char data)
{
	DATA = (((data >> 4) & 0x0F) | EN | RS);
	DATA = (((data >> 4) & 0x0F) | RS);
	
	DATA = ((data & 0x0F) | EN | RS);
	DATA = ((data & 0x0F) | RS);
	
	__delay_us(100);
}

void vLCDsendString(const unsigned char *ucDataString)
{
	while(*ucDataString)	//wait for the final char (0)
	{
		vLCDsendData(*ucDataString++);	//Transmitt the next char
	}
}

//convert a number and print it on LCD
void vLCDsendULong(unsigned long num)
{
	milliard = 0;
	million = 0;
	mil = 0;
	cent = 0;
	diz = 0;
	unit = 0; 

	if(num> 999999) vLCDsendString("Err.");
	else
	{
		milliard = (num / 100000);
		million = (num - (milliard * 100000))/ 10000;
		mil = (num - (milliard * 100000) - (million*10000)) / 1000;
		cent = (num - (milliard * 100000) - (million*10000) - (mil*1000)) / 100;
		diz = (num - (milliard * 100000) - (million*10000) - (mil*1000)-(cent*100)) / 10;
		unit = (num - (milliard * 100000) - (million*10000) - (mil*1000)-(cent*100)-(diz*10));

		if(milliard) vLCDsendData(milliard + 0x30);
		if(num > 9999) vLCDsendData(million + 0x30);
		vLCDsendData(mil + 0x30);
		vLCDsendData(cent + 0x30);
		vLCDsendData(diz + 0x30);
		vLCDsendData(unit + 0x30);
	}
	
	
}	



void vLCDclear(void)
{
	vLCDsendCmd(0x01);
}


void vLCDclearLine(unsigned char ucLine)
{
	unsigned char i = 0x00;

	if (ucLine == 0) vLCDgoto(0,0);
	else vLCDgoto(0,1);
	for(i=0;i++;i<16) vLCDsendData(0x00);
}

//define the position of the cursor (x from 0 to 15, y from 0 to 1)
void vLCDgoto(unsigned char x, unsigned char y)
{	
	unsigned char ucDDRAM_adress = 0x00;
	
	if(x >= NUMBER_OF_CHAR) x = NUMBER_OF_CHAR-1;
	if (y >= NUMBER_OF_LINE) y = NUMBER_OF_LINE-1;

	if(y==0) ucDDRAM_adress = x;
	else if (y==1) ucDDRAM_adress = (x+0x40);
	else ucDDRAM_adress = 0x00;

	vLCDsendCmd(0x80+ucDDRAM_adress);
	
}

//***********************************************
// Advance LCD functions
//***********************************************
// manage the LCD action according to the state

void vManageLCD(void)
{
	switch (ucLCDstate)
	{
		case LOADING:
			vLCDgoto(6,0);
			vLCDsendString("Time");
			vLCDgoto(6,1);
			vLCDsendString("Lapse");
			__delay_ms(1000);
			ucLCDstate = HOME;
			break;

		case HOME:
			vLCDclear();
			vLCDgoto(0,0);
			vLCDsendString("Select Mode:");
			ucLCDstateOld = HOME;
			ucLCDstate = WAIT_FOR_BUTTON;
			break;

		case REMOTE_MODE:
			vLCDclear();
			vLCDsendString("Remote Mode:");
			vLCDgoto(0,1);
			vLCDsendString("Press S to shoot");
			ucLCDstateOld = REMOTE_MODE;
			ucLCDstate = WAIT_FOR_BUTTON;
			break;
		
		case TIME_LAPSE_MODE:
			vLCDclear();
			vLCDsendString("Time Lapse Mode:");
			vLCDgoto(0,1);
			vLCDsendString("Define setting");
			__delay_ms(1000);
			ucLCDstateOld = TIME_LAPSE_MODE;
			ucLCDstate = WAIT_FOR_BUTTON;
			break;
			
		case ABOUT_MODE:
			vLCDclear();
			vLCDsendString("A DIY Project:");
			ucLCDstateOld = ABOUT_MODE;
			ucLCDstate = WAIT_FOR_BUTTON;
			break;

		case WAIT_FOR_BUTTON:
			if (ucLCDstateOld == HOME) vModeSelection();
			if (ucLCDstateOld == REMOTE_MODE) vRemoteMode();
			if (ucLCDstateOld == TIME_LAPSE_MODE) vTimeLapseMode();
			if (ucLCDstateOld == ABOUT_MODE) vAboutMode();
			break;
		
		default: break;
	
	}	//end of switch(ucLCDstate)
}
