//*******************************************************
//*    		lcd.c : source for use of LCD	     		*
//*				LCD1602  16x2char 4bits mode			*
//*						pic16f690		 	  		 	*
//*						MPLAB V8.86				 	    *
//* 					PICC lite 					    *
//* 				Simon Chaudeurge				    *
//* 					August 2012					    *
//*******************************************************

#include "pic.h"
#include "lcd.h"
#include "input.h"
#include "shoot.h"
#include "common.h"
#include "init.h"
#include "modes.h"

unsigned char ucLCDstate = 0x00;
unsigned char ucMode = REMOTE_MODE;



//***********************************************
// user choose the mode 
//	(simple remote or time lapse mode)
//***********************************************
void vModeSelection(void)
{
	__delay_ms(5);
		
	if(ucButtonPressed == SELECT) ucLCDstate = ucMode;
	
	else if(ucButtonPressed == LEFT)
	{
		if(ucMode < ABOUT_MODE) ucMode++;
		else ucMode = REMOTE_MODE;
	}
	
	else if (ucButtonPressed == RIGHT)
	{
		if(ucMode > REMOTE_MODE) ucMode--;
		else ucMode = ABOUT_MODE;
	}

	vLCDgoto(0,1);	
	switch(ucMode)
	{
		case REMOTE_MODE:
			vLCDsendString("< Remote Mode  >");
			break;	
		
		case TIME_LAPSE_MODE:	
			vLCDsendString("<TimeLapse Mode>");
			
			break;
			
		case ABOUT_MODE:	
			vLCDsendString("<  About ...   >");
			break;
			
		default:	//should never happen
			vLCDsendString("<    Error !    >"); 
			while(1);
			break;
	}
			

	
}

//***********************************************
// vRemoteMode()
// 	Shoot witch the camera when user press the SELECT button
// 	return to the main menu when user press LEFT or RIGHT
//***********************************************
void vRemoteMode(void)
{
	if(ucButtonPressed == SELECT) vShoot(); //shoot when SELECT button is pressed
	else if (ucButtonPressed == UP) vFocus();	//focus when LEFT button is pressed
	else if((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT)) ucLCDstate = HOME;	//return to the main menu 
}


//***********************************************
// vAboutMode()
// 	Shoot until user release the SELECT button,
// 	return to the main menu when user press LEFT or RIGHT
//***********************************************
void vAboutMode(void)
{
	unsigned char i = 0;
		if(ucDirection)
		{
			if(Xpos>0) Xpos--;
			else ucDirection=0;
		}
		else
		{
			if(Xpos<(NUMBER_OF_CHAR-5)) Xpos++;
			else ucDirection=1;
		}	
		vLCDgoto(0,1);
		vLCDsendString("                ");
		vLCDgoto(Xpos,1);
		vLCDsendString("V.1.0");
		__delay_ms(300);	
			
		vReadInputSw();
		if((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT))	ucLCDstate = HOME;	//return to the main menu		
}


//***********************************************
// vTimeLapseMode()
// 	Shoot with regal interval time, defined by user,
// 	return to the main menu when user press LEFT or RIGHT
//***********************************************
void vTimeLapseMode(void)
{
	vLCDclear();
	vLCDsendString("Time Lapse Mode:");
	
	//Set the gap time between each shoot (in second)
	vLCDgoto(0,1);
	vLCDsendString("gap (sec) = ");
	do
	{
		vReadInputSw();
		if(ucButtonPressed == UP)
		{
			if(uiShootInterval_sec < 3600) uiShootInterval_sec++;
			else uiShootInterval_sec = 1;
		}
		else if(ucButtonPressed == DOWN)
		{
			if(uiShootInterval_sec > 1) uiShootInterval_sec--;
			else uiShootInterval_sec = 3600;
		}
		else if((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT)) break;
		vLCDgoto(12,1);
		vLCDsendULong(uiShootInterval_sec);
	}
	while(ucButtonPressed != SELECT);

	//Set the total time of the shooting sequence (in minute)
	vLCDgoto(0,1);
	vLCDsendString("Time(min) = ");
	do
	{
		vReadInputSw();
		if(ucButtonPressed == UP)
		{
			if(ulTotalShootTime_min < 3600) ulTotalShootTime_min++;
			else ulTotalShootTime_min = 1;
		}
		else if(ucButtonPressed == DOWN)
		{
			if(ulTotalShootTime_min > 1) ulTotalShootTime_min--;
			else ulTotalShootTime_min = 3600;
		}
		else if((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT)) return;
		vLCDgoto(12,1);
		vLCDsendULong(ulTotalShootTime_min);
	}
	while(ucButtonPressed != SELECT);
	
	//Resum the setting and waiting for confirmation
	vLCDclear();
	vLCDsendString("Please confirm");
	vLCDgoto(0,1);
	vLCDsendString("these settings:");
	__delay_ms(1000);
	vLCDclear();
	
	vLCDsendString("Shoot evry ");
	vLCDsendULong(uiShootInterval_sec);
	vLCDsendString("s");
	vLCDgoto(0,1);
	vLCDsendString("during ");
	vLCDsendULong(ulTotalShootTime_min);
	vLCDsendString(" min");
	do
	{	
		vReadInputSw();
		if((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT)) return;
	}
	while(ucButtonPressed != SELECT);
		
	// the time lapse settings have been confirmed
	ulTotalNumberOfPhoto = ((ulTotalShootTime_min*60) / uiShootInterval_sec);
	ulNumberOfPhotoTaken = 0;
	vDefineShootDelay_sec(uiShootInterval_sec);
	
	vLCDclear();
	vLCDsendString("Shot in progress");
	vLCDgoto(6,1);
	vLCDsendString(" / ");
	vLCDsendULong(ulTotalNumberOfPhoto);
	__delay_ms(1000);
	
	//stay in time lapse mode until shoot sequence timeout
	// or user press on LEFT or RIGHT button to cancel the sequence 
	while((!ucSequenceTimeout) && (!((ucButtonPressed == LEFT) || (ucButtonPressed == RIGHT))))
	{
		vReadInputSw();
		vLCDgoto(0,1);
		vLCDsendULong(ulNumberOfPhotoTaken);
	}
	
	//Quit time lapse mode to the main menu
	vEndOfTimeLapse();						
	vLCDclear();
	vLCDsendString("End of the time");
	vLCDgoto(0,1);
	vLCDsendString("lapse sequence !");
	__delay_ms(1000);
	vLCDclear();
	ucLCDstateOld = TIME_LAPSE_MODE;
	ucLCDstate = HOME;
	return;
}	
