//*******************************************************
//*    		shoot.c : source DSLR action	     		*
//*						pic16f690		 	  		 	*
//*						MPLAB V8.86				 	    *
//* 					PICC lite 					    *
//* 				Simon Chaudeurge				    *
//* 					August 2012					    *
//*******************************************************

#include "pic.h"
#include "common.h"
#include "shoot.h"
#include "lcd.h"
#include "init.h"
#include "modes.h"

unsigned int uiShootDelaySec = 1;
unsigned int uiTempShootDelaySec = 0;
unsigned short ucManualPostscaler = 0x00;


//***********************************************
// vShoot function : Trig a photo shoot
//***********************************************
void vShoot(void)
{
	unsigned char i = 0;
	SHOOT=1;
	if(ucLCDstateOld == TIME_LAPSE_MODE)	__delay_ms(500);
	else __delay_ms(2);
	SHOOT=0;
	if(ulNumberOfPhotoTaken != 999999) ulNumberOfPhotoTaken++;
}

//***********************************************
// vFocus function : Trig a photo focus
// Note: for this first version, Focus is wired with shoot
//  so Focus() function is useless
//***********************************************
void vFocus(void)
{
	FOCUS=1;
	__delay_ms(2);
	FOCUS=0;
}

//***********************************************
// vDefineShootDelay function : 
//	description: Define the delay (in second)
//		between each shot (from 1sec to 1hour)
//	inputs:	delay(second)
//	outputs: None
//***********************************************
void vDefineShootDelay_sec(unsigned int uiDelay)
{
	if(uiDelay == 0) uiShootDelaySec = 1;
	else uiShootDelaySec = uiDelay;
	
	uiTempShootDelaySec = 0;
	ucSequenceTimeout = 0;
	TMR2 = 0x00; //Reset Timer2
	vEnableTimerInterrupt();
}


//***********************************************
// Interruption vector
//***********************************************
static void interrupt isr(void)
{
//	Period = 32,256ms
//	Period * 31 = 999.936ms  (as close as possible to 1seconde)
	if(TMR2IF)
	{
		//Manual Poscaler to get an interrupt every 1sec
		if(ucManualPostscaler != 0xFF) ucManualPostscaler++;
		if(ucManualPostscaler >= MANUAL_POSTCALER)
		{
			ucManualPostscaler = 0x00;
			
			//Tempo to defined the delay of interrupt (in seconde)
			if (uiTempShootDelaySec != 0xFF) uiTempShootDelaySec++;
			if (uiTempShootDelaySec >= uiShootDelaySec)
			{
				vShoot();
				uiTempShootDelaySec = 0;
				
				//define if the total number of photo taken has been reached
				if (ulNumberOfPhotoTaken >= ulTotalNumberOfPhoto) vEndOfTimeLapse();
				else ucSequenceTimeout = 0;
			}
		}			
		TMR2 = 0x00;	//reset Timer2
		TMR2IF = 0;
	}
} 

//***********************************************
// End of Time Lapse Sequence : disable Timer and reset tempo
//***********************************************
void vEndOfTimeLapse(void)
{
	ucSequenceTimeout = 1;
	vDisableTimerInterrupt();
	ulNumberOfPhotoTaken = 0;
}